/*
 Get acceleration data from Texas Instruments Chronos watch (eZ430).
 Draw X, Y & Z and apply Kalman filter on each axis.
 
 Original code by Sean Brewer and Oliver Smith
 http://chemicaloliver.net/programming/graphing-ti-ez430-chronos-watch-data-in-processing/
 http://e2e.ti.com/support/microcontrollers/msp43016-bit_ultra-low_power_mcus/f/166/t/32714.aspx

 Excellent Kalman information found at Marcus's site
 http://interactive-matter.org/2009/12/filtering-sensor-data-with-a-kalman-filter/
 
 Additions by Michel Blanc.
 Copyright (c) 2010 Sean Brewer / Oliver Smith / Michel Blanc
 
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.

If you want you may contact me at mblanc@erasme.org
or on jabber: leucos@jabber.org
*/

import controlP5.*;

ControlP5 controlP5;

// Kalman objects for X, Y and Z axis
Kalman kx;
Kalman ky;
Kalman kz;

// Watch object
Ez430 watch;

// Initial Kalman values
float q = 0.0625;
float r = 32.0;
float p = 1.3833;

// Holders for previous x,y,z
// Previous value is kept for graphing purpose
float prevx, prevy, prevz;

// Holds kalmanified x,y,z values
// Previous value is kept for graphing purpose
float prevkx, prevky, prevkz;

// Current graph position on X axis
int xPos = 1;

// Screen size unit (one yunit per axis, so the screen size is 3 * yunit)
int xunit = 400;
int yunit = 300;

PFont font;

// ControP5 Checkbox holds the two checkboxes
CheckBox checkbox;

// 2 sliders per axis for Kalman parameters
Slider sxr,sxq,syr,syq,szr,szq;

// Do we show sliders
boolean showSliders = true;
boolean syncSliders = true;

void setup() {
  //Draw window
  size(xunit, 3*yunit);
  background(0);

  smooth();

  font = loadFont("Univers45.vlw.gz"); 
  textFont(font); 

  controlP5 = new ControlP5(this);
  
  // Sliders for X
  sxq = controlP5.addSlider("xqvalue",0,4,q,20,yunit-140,10,100);
  sxq.setLabel("q value");
  
  sxr = controlP5.addSlider("xrvalue",0,40,r,60,yunit-140,10,100);
  sxr.setLabel("r value");

  // Sliders for Y
  syq = controlP5.addSlider("yqvalue",0,4,q,20,2*yunit-140,10,100);
  syq.setLabel("q value");
  
  syr = controlP5.addSlider("yrvalue",0,40,r,60,2*yunit-140,10,100);
  syr.setLabel("r value");

  // Sliders for Z
  szq = controlP5.addSlider("zqvalue",0,4,q,20,3*yunit-140,10,100);
  szq.setLabel("q value");
  
  szr = controlP5.addSlider("zrvalue",0,40,r,60,3*yunit-140,10,100);
  szr.setLabel("r value");
  
  // Checkboxes
  checkbox = controlP5.addCheckBox("ctrlCheckBox",20,20);  
  checkbox.addItem("show sliders",0);
  checkbox.addItem("sync sliders",10);
  checkbox.activate(0);
  checkbox.activate(1);
  
  watch = new Ez430(this, "/dev/ttyACM0");
  watch.debug(false);
  
  initKalman();
  blankscreen();
} 

void controlEvent(ControlEvent theEvent) {
  if(theEvent.isGroup() && theEvent.group().name() == "ctrlCheckBox") {
    // checkbox uses arrayValue to store the state of 
    // individual checkbox-items. usage:
    for(int i=0;i<theEvent.group().arrayValue().length;i++) {
      int n = (int)theEvent.group().arrayValue()[i];
      print(n);
      if (i == 0) {
        showSliders = theEvent.group().arrayValue()[i] == 0.0 ? false : true;
        sxr.setVisible(showSliders);
        sxq.setVisible(showSliders);
        syr.setVisible(showSliders);
        syq.setVisible(showSliders);
        szr.setVisible(showSliders);
        szq.setVisible(showSliders);
        print("changing sliders");
        blankscreen();
      } else {
        syncSliders = (theEvent.group().arrayValue()[i] == 0.0 ? false : true);
        /*syr.setBroadcast(false);
        syq.setVisible(!syncSliders && showSliders);
        szr.setVisible(!syncSliders && showSliders);
        szq.setVisible(!syncSliders && showSliders);       */
      }
    }
  }
  println();
}

void blankscreen() {
  background(0);
  stroke(255,255,255);
  strokeWeight(2);
  noFill();
  rect(0,0,400,yunit);
  rect(0,yunit,xunit,2*yunit);
  rect(0,2*yunit,xunit,3*yunit);
  
  fill(200,200,200);
  text("X", xunit-50, 0*yunit+50);
  text("Y", xunit-50, 1*yunit+50);     
  text("Z", xunit-50, 2*yunit+50);
}

void initKalman() {
  print("Waiting for watch...");
  while(watch.Available() == 0) {};
  print("ok\nReading initial values...");
  watch.ReadValues();
  print ("ok\nInitializing kalman filters...");
  
  kx = new Kalman (q, r, p, watch.x());
  ky = new Kalman (q, r, p, watch.y());
  kz = new Kalman (q, r, p, watch.z());
  println("ok");
}

void xrvalue(float value) {
  kx.setR(value); 
  println("a slider event. setting xR to "+value);
  
  if (syncSliders) {
    ky.setR(value); 
    kz.setR(value);  
   
    syr.setValue(value);
    szr.setValue(value);
  }
}

void xqvalue(float value) {
  kx.setQ(value); 
  println("a slider event. setting xQ to "+value);

  if (syncSliders) {
    ky.setQ(value); 
    kz.setQ(value);
    
    syq.setValue(value);
    szq.setValue(value);    
  }  
}

void yrvalue(float value) {
  ky.setR(value); 
  println("a slider event. setting yR to "+value);
}

void yqvalue(float value) {
  ky.setQ(value); 
  println("a slider event. setting yQ to "+value);
}

void zrvalue(float value) {
  kz.setR(value); 
  println("a slider event. setting zR to "+value);
}

void zqvalue(float value) {
  kz.setQ(value);  
  println("a slider event. setting zQ to "+value);
}

void mousePressed() {
  if (mouseButton == RIGHT) {
    blankscreen();
  }
}
  
void draw() {
  if ((watch.Available() > 0) && watch.ReadValues()) {
    strokeWeight(1);

    // X
    stroke(127,64,64);

    //draw x line
    float x = map(watch.x(), -128, 128, 0, yunit);
    line(xPos-1, yunit - prevx, xPos, yunit - x);

    // draw Kalman for x
    kx.Update(watch.x());
    stroke(187,124,124);      
    float x2 = map(kx.Value(), -128, 128, 0, yunit);
    line(xPos-1, yunit - prevkx, xPos, yunit - x2);
      
    // Y
    stroke(127,64,127);

    //draw y line
    float y = map(watch.y(), -128, 128, 0, yunit);
    line(xPos-1, 2*yunit - prevy, xPos, 2*yunit - y);
       
    // draw Kalman for y
    ky.Update(watch.y());
    stroke(187,124,187);      
    float y2 = map(ky.Value(), -128, 128, 0, yunit);
    line(xPos-1, 2*yunit - prevky, xPos, 2*yunit - y2);
      
    // Z
    stroke(64,64,127);
      
    //draw z line
    float z = map(watch.z(), -128, 128, 0, yunit);
    line(xPos-1, 3*yunit - prevz, xPos, 3*yunit - z);

    // draw Kalman for z
    kz.Update(watch.z());
    stroke(124,124,187);      
    float z2 = map(kz.Value(), -128, 128, 0, yunit);
    line(xPos-1, 3*yunit - prevkz, xPos, 3*yunit - z2);

    // We keep previous values to draw lines      
    prevx = x;
    prevy = y;
    prevz = z;
      
    // Same for the kalman values
    prevkx = x2;
    prevky = y2;
    prevkz = z2;
      
    //if xpos has reached end of window start again at 0
    if (xPos >= width) {
      xPos = 0;
      blankscreen();
    } else {
      // increment the horizontal position:
      xPos++;
    }
  }    
}

